* SK8
* LP9
**************************************************
* Remote debugger for the TI-99/4A     PIO version 1.02
* --------------------------------
* c) 2000, Thierry Nouspikel
**************************************************
       AORG   >A000                              must be assembled here, but can run anywhere
	PSEG	T 
CRU232	EQU	>1300		                     CRU of our RS232 card
*==================================================
* Main entry point, used when loading the program
* The program is dynamically relocatable: it can be moved
* around in memory, by just updating R9 (new - old address)
*==================================================
TOP	NOP			                     ; room for single-step execution
	NOP
       NOP
START	BLWP	@START1		              initial entry point (needs patching if moved)
SIZE   DATA   ENDPG-TOP                          size of this program: >0180
START1	DATA	WREGS,PUMP12	                     to message pump (needs patching if moved)
BREAK	DATA	WREGS,BREAK1	                     to breakpoint entry (ditto)
* >A014
*--------------------------------------------------
* Entry from breakpoint (XOP1=>FFD8>FFF8, XOP2 = >83A0>8300, XOP6 >8300>C342)
* Will contain a BLWP @BREAK
*--------------------------------------------------
BREAK1	MOV    R13,R8                             XOP workspace
       AI     R8,24                              point at R12
       MOV    *R8+,R12                           copy into our R12
       MOV    *R8+,R13                           ditto for return params
       MOV    *R8+,R14
       MOV    *R8,R15
      
	MOV    R12,R8                             to pass in WHERE command
	SBZ	0		                     turn peripheral card off (if any)
* >A026       		                     continue below
*--------------------------------------------------
* Find where we are in memory, return R9-R15
* R8: debugged program R12
* R9: offset between intended and actual address
* R10: workspace of debugger + 21 (i.e. R10 lsb)
* R11: not used (LP6+2)
* R12: RS232 card CRU
* R13: worskpace of debugged program
* R13: PC of debugged program 
* R13: status of debugged program
*--------------------------------------------------
WHERE	LI	R6,>045B	                     this is B *R11
	BL	R6		                     find where we are
HERE	LI	R9,TOP-HERE->A000    	        R11 should contain HERE, unless moved
	A	R11,R9		                     calculate load offset

	STWP   R10                                point at our workspace
	LI	R6,>0010	                     transfer 8 registers (R8-R15)
	A      R6,R10                             point at R8
	LI	R2,>0001	                     next byte
	LI	R12,CRU232	                     CRU address of RS232 card
	SBO	0		                     turn RS232 card on
	SBO	2		                     init handshake out line
	JMP	LP6		                     transfer regs to remote machine
* >A048
*--------------------------------------------------
* CRU write
*--------------------------------------------------
SBO0	CLR    R2		                     SBO: will be >1D00
SBZ0	AI	R2,>001D	                     SBZ: will be >1E00
	SLA    R2,8
	BL	@GETDAT(R9)	                     get CRU address
	MOV	R1,R12		                     set it
	X	R2		                     execute SBO or SBZ
	JMP	PUMP12		                     restore CRU and get next message
* >A05A
*--------------------------------------------------
* CRU read
*--------------------------------------------------
TB0	BL	@GETDAT(R9)	                     get CRU address
	MOV	R1,R12		                     set it
	TB	0		                     test bit
	JEQ	SK1		                     R2 contains >0001
	SETO	R2		                     now >FFFF
SK1	STWP   R10                                point at our workspace
       C      *R10+,*R10+	                     source = R2
       LI     R12,CRU232
	BL	@PUTBY(R9)	                     send msb
* 	JMP	PUMP12		                     restore CRU and get next message
* >A072
*--------------------------------------------------
* Main message pump
* Messages are passed from the remote machine as 1 word
* containing the offset of the desired procedure (0 through 28)
*--------------------------------------------------
PUMP12	LI	R12,CRU232	                     restore CRU address of RS232 card
	SBO	0		                     turn RS232 card on

PUMP	SBO	2		                     reset handshake
	LI     R6,>0002	                     default: 2 bytes
	LI	R2,>0001	                     default: cpu mem
	CLR	R4		                     default: no peripheral card
	BL	@GETDAT(R9)	                     get message into R1
	A	R9,R1		                     adjust address
	MOV	@VECTS(R1),R11	              get vector
	A	R9,R11		                     adjust address
	B	*R11		                     call routine
* >A092
*--------------------------------------------------
* Read from TI-99/4A memory
*--------------------------------------------------
READC	BL	@GETDAT(R9)	                     get card CRU
	MOV	R1,R4		                     save it
	JMP	READM
READD	CLR	R2		                     memory mapped device: keep same address
READM	BL	@GETDAT(R9)	                     read many bytes: get # of bytes
	MOV	R1,R6		                     save it
READ2	BL	@GETDAT(R9)	                     read 2 bytes: get address
	MOV	R1,R10 		              save source	
LP6	BL	@PUTBY(R9)	                     send a byte to remote machine
	A	R2,R10		                     increment pointer (or not)
	DEC	R6		                     next byte
	JGT	LP6		                     more to do
	JMP    PUMP		                     done
* >A0B4
*--------------------------------------------------
* Write to TI-99/4A memory
*--------------------------------------------------	
WRITEC BL	@GETDAT(R9)	                     get card CRU
	MOV	R1,R4		                     save it
	JMP	WRITEM
WRITED	CLR	R2		                     memory mapped device: keep same address
WRITEM	BL	@GETDAT(R9)	                     write many bytes: get # of bytes
	MOV	R1,R6		                     save it
WRITE2	BL	@GETDAT(R9)	                     write 2 bytes: get address
	MOV	R1,R10		                     save destination
LP5	BL	@GETBY(R9)	                     get a byte from remote machine
	A	R2,R10		                     increment pointer (or not)
	DEC	R6		                     next byte
	JNE	LP5		                     more to do
	JMP	PUMP		                     wait for next message
* >A0D6
*--------------------------------------------------
* Run program on the TI-99/4A or return to caller
*--------------------------------------------------	
RUNC	BL	@GETDAT(R9)	                     get CRU of peripheral card
	MOV	R1,R4		                     save it
RUN	BL	@GETDAT(R9)	                     get address
	MOV	R1,R14		                     will become PC after RTWP
RTWP0	SBZ	0		                     turn RS232 card off
	MOV	R4,R12		                     get peripheral card CRU
	JEQ	SK3		                     none
	SBO	0		                     turn card on
SK3	RTWP           	
* >A0EC
*--------------------------------------------------	
* Get a word from remote machine
*--------------------------------------------------
GETDAT	MOV    R11,R3                             save return point
       STWP   R10                                point at workspace
	INCT	R10    	                     destination: R1
	BL	@GETBY(R9)	                     get msb 
	INC	R10
	BL	@GETBY(R9)	                     get lsb
	B	*R3
* >A0FE
*--------------------------------------------------	
* Get a byte from remote machine
*--------------------------------------------------	
GETBY	SBO	1		                     put PIO in read mode
	CLR	R5
LP1	DEC	R5		                     timeout counter
	JEQ	PUMP		                     expired
	TB	2		                     wait for call
	JNE	LP1

	SBZ	2		                     shake hands
	CLR	R5
LP2	DEC	R5		                     timeout counter
	JEQ	PUMP		                     expired
	TB	2		                     wait for answer
	JEQ	LP2

	MOVB	@>5000,R5	                     receive byte
	SBZ	0		                     turn RS232 off
	MOV	R4,R4		                     peripheral card?
	JEQ	SK5		                     no
	MOV	R4,R12		                     foreing card CRU
	SBO	0		                     card on
SK5	MOVB	R5,*R10		              copy byte
	SBZ	0		                     card off
	LI	12,CRU232	                     reload our CRU
	SBO	0		                     turn RS232 on
SK6	SBO	2		                     acknowledge
	B	*R11
* >A132
*--------------------------------------------------	
* Send a byte to remote machine
*--------------------------------------------------	
PUTBY	SBZ	1		                     put PIO in write mode
	CLR	R5
LP7	DEC	R5		                     timeout counter
 	JEQ	PUMP		                     expired
	TB	2		                     wait for remote to be idle
	JEQ	LP7

	SBZ	0		                     turn RS232 off
	MOV	R4,R4		                     peripheral card?
	JEQ	SK7		                     no
	MOV	R4,R12		                     foreing card CRU
	SBO	0		                     card on
SK7	MOVB	*R10,R5		              get byte
	SBZ	0		                     card off
	LI	12,CRU232	                     reload our CRU
	SBO	0		                     turn RS232 on
	MOVB	R5,@>5000	                     send byte

	SBZ	2		                     signal byte sent
	CLR	R5		
LP8	DEC	R5		                     timeout counter
	JEQ	PUMP		                     expired
	TB	2		                     wait for acknowledge
	JNE	LP8
	JMP	SK6		                     reset handshake and return
* >A164
*--------------------------------------------------
* Data area
*--------------------------------------------------	
VECTS	DATA	READ2		                     0 : read two bytes
	DATA	READM		                     2 : read many bytes
	DATA	READD		                     4 : read from same address
	DATA	READC		                     6 : read peripheral card
	DATA	WRITE2		                     8 : write two bytes
	DATA	WRITEM		                     10: write many bytes
	DATA	WRITED		                     12: write to same address
	DATA	WRITEC		                     14: write peripheral card
	DATA	SBZ0		                     16: SBZ
	DATA	SBO0		                     18: SBO
	DATA	TB0		                     20: Test CRU bit
	DATA	RUN		                     22: Run program
	DATA	RUNC		                     24: Run program in peripheral card
	DATA	RTWP0		                     26: Return to caller
	DATA	WHERE		                     28: Send R9-R15
ENDPG
* >A182                                          default workspace
WREGS	DATA	0,0,0,0,0,0,0,0,0,0,0,0,0,TOP,WREGS,ENDPG

	END
